#--
# ANSIColor
#
# Copyright (c) 2002-2005 Florian Frank
#
# GNU Lesser General Public License
#
# This program is free software.
# You can distribute/modify this program under the terms of
# the GNU Lesser General Public License version 2 or later.
#
#
# ==========================================================================
# Revision History ::
# --------------------------------------------------------------------------
#  2005.04.28  Trans  * Minor modifications to documentation.
#
#  2004-11-30  Trans  * Ported to Facets collection (transami)
#
#  2004-09-28  FF     * First release on Rubyforge
#                     * Supports Rubygems now
#
#  2003-10-09  FF     * Added uncolored method as suggested by
#                       Thomas Husterer <Thomas.Husterer@heidelberg.com>
#                     * Added attribute methods with string arguments
#                     * Deleted now unused files
#
#  2002-07-27  FF     * Minor Code Cleanup
#                     * Added cdiff.rb
#
#  2002-06-12  FF     * Initial Release
# ==========================================================================
#++

#:title: ANSIColor
#
# Module which makes it very easy to use ANSI codes.
# These are esspecially nice for beautifying shell output.
#
# == Synopsis
#
#   require 'mega/ansicolor'
#
#   include ANSIColor
#
#   p red, "Hello", blue, "World"   
#   => "\e[31mHello\e[34mWorld"
#
#   p red { "Hello" } + blue { "World" }
#   => "\e[31mHello\e[0m\e[34mWorld\e[0m"
#
# == Supported ANSI Commands
#
#     clear
#     reset           # synonym for :clear
#     bold
#     dark
#     italic          # not widely implemented
#     underline
#     underscore      # synonym for :underline
#     blink
#     rapid_blink     # not widely implemented
#     negative        # no reverse because of String#reverse
#     concealed
#     strikethrough   # not widely implemented
#     black
#     red
#     green
#     yellow
#     blue
#     magenta
#     cyan
#     white
#     on_black
#     on_red
#     on_green
#     on_yellow
#     on_blue
#     on_magenta
#     on_cyan
#     on_white
#
# == Author(s)
#
# * Florian Frank <flori#ping.de>
#

module ANSIColor

  @@attributes = [
    [ :clear        ,   0 ], 
    [ :reset        ,   0 ],     # synonym for :clear
    [ :bold         ,   1 ], 
    [ :dark         ,   2 ], 
    [ :italic       ,   3 ],     # not widely implemented
    [ :underline    ,   4 ], 
    [ :underscore   ,   4 ],     # synonym for :underline
    [ :blink        ,   5 ], 
    [ :rapid_blink  ,   6 ],     # not widely implemented
    [ :negative     ,   7 ],     # no reverse because of String#reverse
    [ :concealed    ,   8 ], 
    [ :strikethrough,   9 ],     # not widely implemented
    [ :black        ,  30 ], 
    [ :red          ,  31 ], 
    [ :green        ,  32 ], 
    [ :yellow       ,  33 ], 
    [ :blue         ,  34 ], 
    [ :magenta      ,  35 ], 
    [ :cyan         ,  36 ], 
    [ :white        ,  37 ], 
    [ :on_black     ,  40 ], 
    [ :on_red       ,  41 ], 
    [ :on_green     ,  42 ], 
    [ :on_yellow    ,  43 ], 
    [ :on_blue      ,  44 ], 
    [ :on_magenta   ,  45 ], 
    [ :on_cyan      ,  46 ], 
    [ :on_white     ,  47 ], 
  ]

  @@attributes.each do |c, v|
    eval %Q{
        def #{c.to_s}(string = nil)
            result = "\e[#{v}m"
            if block_given?
                result << yield
                result << "\e[0m"
            elsif string
                result << string
                result << "\e[0m"
            elsif respond_to?(:to_str)
                result << self
                result << "\e[0m"
            end
            return result
        end
    }
  end

  ColoredRegexp = /\e\[([34][0-7]|[0-9])m/

  def uncolored(string = nil)
    if block_given?
      yield.gsub(ColoredRegexp, '')
    elsif string
      string.gsub(ColoredRegexp, '')
    elsif respond_to?(:to_str)
      gsub(ColoredRegexp, '')
    else
      ''
    end
  end

  module_function

  def attributes
    @@attributes.map { |c| c[0] }
  end

end #module ANSIColor



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#

=begin testing

  require 'test/unit'

  class TC_ANSIColor < Test::Unit::TestCase

    include ANSIColor

    def test_methods
      str = red + "Hello" + blue + "World"
      out = "\e[31mHello\e[34mWorld"
      assert_equal( out, str )
    end

    def test_blocks
      str = red { "Hello" } + blue { "World" }
      out = "\e[31mHello\e[0m\e[34mWorld\e[0m"
      assert_equal( out, str )
    end

  end

=end
