#--
# Association
#
# Copyright (c) 2005 Thmas Sawyer
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
# ==========================================================================
# Revision History ::
# --------------------------------------------------------------------------
#  2005-10-25  Trans     * Initial Release
# ==========================================================================
#++

#:title: Association
#
# Gerenal binary association, useful for creating ordered hashes,
# or mixed collections.
#
# The method :>> is used to construct the association. It is an uncommon
# method, but wehere is has been used prior a suitable alias has been
# made, #rshift. For Date class it has ben named #months_later.
#
# == Synopsis
#
#   c = [ :a >> 1, :b >> 2 ]
#
#   c.each { |k,v| puts "#{k} associated with #{v} }
#
# _produces_
#
#   a associated with 1
#   b associated with 2
#
# == Author(s)
#
# * Thomas Sawyer
#

class Association

  include Comparable

  attr_accessor :index, :value

  def self.[](*args)
    new(*args)
  end

  def initialize(index, value=nil)
    @index = index
    @value = value
  end

  def <=>(assoc)
     return -1 if self.value < assoc.value
     return 1 if self.value > assoc.value
     return 0 if self.value == assoc.value
  end

  def invert!
    temp = @index
    @index = @value
    @value = temp
  end

  def to_s
    return "#{index.to_s}#{value.to_s}"
  end

  def inspect
    %{#{@index.inspect} >> #{@value.inspect}}
  end

  def to_ary
    [ @index, @value ]
  end

end

# Setup the >> method, aliasing where already used.

class Bignum
  alias_method( :rshift, :>>) if method_defined?(:>>)
end

class Fixnum
  alias_method( :rshift, :>>) if method_defined?(:>>)
end

class Date
  alias_method( :months_later, :>>) if method_defined?(:>>)
end

class IPAddr
  alias_method( :rshift, :>>) if method_defined?(:>>)
end

class Process::Status
  alias_method( :rshift, :>>) if method_defined?(:>>)
end

class Object
  def >>(to)
    Association.new(self, to)
  end
end


#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
# TODO
=begin #test

  require 'test/unit'

  class TC01 < Test::Unit::TestCase

complex_hierarchy = [
  'parent' >> 'child',
  'childless',
  'another_parent' >> [
    'subchildless',
    'subparent' >> 'subchild'
  ]
]

puts complex_hierarchy.inspect


# ordered hash

ohash = [ 'A' >> '3', 'B' >> '2', 'C' >> '1' ]

puts ohash.inspect

puts
ohash.each { |e1,e2| puts "#{e1} ~> #{e2}" }
puts

ohash.sort!
puts ohash.inspect

ohash.each { |a| a.invert! }

puts ohash.inspect
ohash.sort!
puts ohash.inspect


# complex hierarchies

complex = [ 'Drop Menu' >> [ 'Button 1', 'Button 2', 'Button 3' ],
            'Help'
          ]

puts complex.inspect

complex[0].index = 'DropDown Menu'

puts complex.inspect

=end
