#--
# Class Inherit
#
# Copyright (c) 2005 Thomas Sawyer
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
#
# ==========================================================================
# Revision History ::
# --------------------------------------------------------------------------
#  05.10.13  trans   * Fixed bug with >2 deep module inheritance.
#  05.10.10  trans   * First Edition
# ==========================================================================
#++

#:title: Class Inherit
#
# Miniframework provides a very convenient way to have modules
# pass along class methods in the inheritance chain.
#
# == Synopsis
#
#   module M
#     class_inherit {
#       def x ; 4 ; end
#     }
#   end
#
#   class C
#     include M
#   end
#
#   M.x  #=> 4
#   C.x  #=> 4
#
# == Notes
#
# Just a quick comment on the need for this behavior.
#
# A module is an encapsulation of code, hence when a module is included (or extends),
# the module itself should have discretion over how it effects the receiving class/module.
# That is the very embodiment of encapsulation. Having it otherwise, as Ruby now does,
# stymies the practice --and we end up with "hacks", like this and ClassMethods, to compensate.
#
# Ruby would be much improved by making this bevaivor standard. And making non-inheritance
# the exception, which is alwasy easy enough to achieve: put the code in a separate
# (and thus uninherited) module.
#
# == Author(s)
#
# * Thomas Sawyer
#
class Module

  alias_method :append_features_no_class_inherit, :append_features
  def append_features(base)
    result = append_features_no_class_inherit( base )
    class_inherit.each do |ext|
      base.extend( ext )
      unless base.is_a?(Class)
        bci = base.class_inherit
        bci << ext unless bci.include?(ext)
      end
    end
    result
  end

  def class_inherit(mod=nil,&blk)
    return (@_class_inherit||=[]) unless blk
    unless mod.is_a?(Module)
#      raise ArgumentError, "missing module or block parameter" unless block_given?
      mod = Module.new(&blk)
    end
    @_class_inherit ||= []
    @_class_inherit << mod
#    anc = ancestors[1]
    extend( mod )
    mod
  end

end

class Class
  undef_method :class_inherit
end



#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#

=begin testing

  require 'test/unit'

  class TC01 < Test::Unit::TestCase
    module M
      class_inherit do
        def x ; 4 ; end
      end
    end
    module N ; include M ; end
    module O ; include N ; end
    module P
      include M
      include O
    end

    def test_01_001
      assert_equal( 4, M.x )
    end
    def test_01_002
      assert_equal( 4, N.x )
    end
    def test_01_003
      assert_equal( 4, O.x )
    end
    def test_01_003
      assert_equal( 4, P.x )
    end
  end

  class TC02 < Test::Unit::TestCase
    module M
      class_inherit do
        def x ; 4 ; end
      end
    end
    module N ; include M ; end
    module O ; include N ; end
    module P
      include O
      class_inherit do
        def x ; super + 6 ; end
      end
    end
    class C ; include M ; end
    class D < C ; end
    class F
      include M
      def self.x ; super + 4 ; end
    end
    class G
      include O
      def self.x ; super + 5 ; end
    end

    def test_01_001
      assert_equal( 10, P.x )
    end
    def test_01_002
      assert_equal( 4, C.x )
    end
    def test_01_003
      assert_equal( 4, D.x )
    end
    def test_01_004
      assert_equal( 8, F.x )
    end
    def test_01_005
      assert_equal( 9, G.x )
    end
  end

  class TC03 < Test::Unit::TestCase
    module Q
      class_inherit do
        def x
          @x ||= []
          if ancestors[1]
            ancestors[1].x + @x
          else
            @x
          end
        end
        def x! ; @x ||= [] ; end
      end
    end
    class R
      include Q
    end

    def test_03_001
      Q.x! << 1
      assert_equal( [1], Q.x )
    end
    def test_03_002
      R.x! << 2
      assert_equal( [1,2], R.x )
    end
    def test_03_003
      assert_equal( [1], Q.x )
    end
  end

=end
